



module i2c_master_cfgport (
	cfg_clk_i, 
	cfg_rstn_i, 
	cfg_wren_i, 
	cfg_addr_i, 
	cfg_rden_i,
	cfg_wdat_i, 
	cfg_rdat_o, 
	cfg_rdat_vld_o, 
	cfg_wdat_rdy_o, 
	cfg_inta_o,
    scl_io,
    sda_io
);

	input        cfg_clk_i;     // master clock input
	input        cfg_rstn_i;     // synchronous active high reset
	input        cfg_wren_i;     // lower address bits
	input  [2:0] cfg_addr_i;     // databus input
	input        cfg_rden_i;     // databus output
	input  [7:0] cfg_wdat_i;      // write enable input
	output [7:0] cfg_rdat_o;     // stobe/core select signal
	output       cfg_rdat_vld_o;     // valid bus cycle input
	output       cfg_wdat_rdy_o;     // bus cycle acknowledge output
	output       cfg_inta_o;    // interrupt request signal output
    inout        scl_io;
    inout        sda_io;

reg cfg_wdat_rdy_o = 0;
reg cfg_rdat_vld_o = 0;

reg r1_cfg_wren;
reg r2_cfg_wren;

reg r1_cfg_rden;
reg r2_cfg_rden;

always @(posedge cfg_clk_i)
begin
    r1_cfg_wren <= cfg_wren_i;
    r2_cfg_wren <= r1_cfg_wren;
    cfg_wdat_rdy_o <= r1_cfg_wren == 1 && r2_cfg_wren == 0;

    r1_cfg_rden <= cfg_rden_i;
    r2_cfg_rden <= r1_cfg_rden;
    cfg_rdat_vld_o <= r1_cfg_rden == 1 && r2_cfg_rden == 0;
end








	I2C_MASTER_Top U_I2C_MASTER_Top(
		.I_CLK(cfg_clk_i), //input I_CLK
		.I_RESETN(cfg_rstn_i), //input I_RESETN
		.I_TX_EN(cfg_wren_i & cfg_wdat_rdy_o), //input I_TX_EN
		.I_WADDR(cfg_addr_i), //input [2:0] I_WADDR
		.I_WDATA(cfg_wdat_i), //input [7:0] I_WDATA
		.I_RX_EN(cfg_rden_i), //input I_RX_EN
		.I_RADDR(cfg_addr_i), //input [2:0] I_RADDR
		.O_RDATA(cfg_rdat_o), //output [7:0] O_RDATA
		.O_IIC_INT(cfg_inta_o), //output O_IIC_INT
		.SCL(scl_io), //inout SCL
		.SDA(sda_io) //inout SDA
	);


endmodule

