
`timescale 1ns / 1ps

module apb_master_model #(
	parameter ADDR_WIDTH = 16,
	parameter DATA_WIDTH = 32
)(
	input                                 apb_clk_i ,    
	input                                 apb_rstn_i,   
	output reg [ADDR_WIDTH-1:0]           apb_addr_o,  
	output reg                            apb_sel_o,
	output reg                            apb_ena_o,
	output reg                            apb_wr_o,    
	  
	output reg [DATA_WIDTH-1:0]           apb_wdata_o,    
	input      [DATA_WIDTH-1:0]           apb_rdata_i,    
	input                                 apb_rdy_i,
	output reg [DATA_WIDTH/8-1:0]         apb_strb_o  
);


////////////////////////////////////////////////////////////////////
//
// Memory Logic
//

initial
	begin
		apb_addr_o  = {ADDR_WIDTH{1'b0}};
		apb_wdata_o = {DATA_WIDTH{1'b0}};
		apb_sel_o   = 1'b0;
		apb_ena_o   = 1'b0;
		apb_wr_o    = 1'b0;
		apb_strb_o  = {{DATA_WIDTH/4}{1'b0}};
		//apb_wren_o  = 1'h0;
		//apb_rden_o  = 1'h0;
		#1;
		$display("\nINFO: APB MASTER MODEL INSTANTIATED (%m)\n");
	end

////////////////////////////////////////////////////////////////////
//
// CFG write cycle
//

task apb_write;
	input   delay;
	integer delay;

	input	[ADDR_WIDTH -1:0]	a;
	input	[DATA_WIDTH -1:0]	d;

	begin
		repeat(delay) @(posedge apb_clk_i);
		#1;
		apb_addr_o   = a;
		apb_wdata_o  = d;
		apb_sel_o   = 1'b1;
		apb_ena_o   = 1'b1;
		apb_wr_o    = 1'b1;
		apb_strb_o  = 4'hf;
		@(posedge apb_clk_i);
		while(~apb_rdy_i)	@(posedge apb_clk_i);
		#1;
		apb_addr_o  = {ADDR_WIDTH{1'b0}};
		apb_wdata_o = {DATA_WIDTH{1'b0}};
		apb_sel_o   = 1'b0;
		apb_ena_o   = 1'b0;
		apb_wr_o    = 1'b0;
		apb_strb_o  = 4'h0;
		@(posedge apb_clk_i);
		@(posedge apb_clk_i);

	end
endtask


task apb_read;
	input   delay;
	integer delay;

	input	[ADDR_WIDTH -1:0]	a;
	output	[DATA_WIDTH -1:0]	d;

	begin

		repeat(delay) @(posedge apb_clk_i);

		#1;
		apb_addr_o  = a;
		apb_wdata_o = {DATA_WIDTH{1'b0}};
		apb_sel_o   = 1'b1;
		apb_ena_o   = 1'b1;
		apb_wr_o    = 1'b0;

		@(posedge apb_clk_i);

		while(~apb_rdy_i)	@(posedge apb_clk_i);

		#1;
		apb_addr_o  = {ADDR_WIDTH{1'b0}};
		apb_wdata_o  = {DATA_WIDTH{1'b0}};
		apb_sel_o   = 1'b0;
		apb_ena_o   = 1'b0;
		apb_wr_o    = 1'b0;
		d           = apb_rdata_i;
		@(posedge apb_clk_i);
		@(posedge apb_clk_i);
		

	end
endtask

reg [DATA_WIDTH   -1:0] q;

task apb_cmp;
	input   delay;
	integer delay;

	input [ADDR_WIDTH -1:0]	a;
	input [DATA_WIDTH -1:0]	d_exp;

	begin
		apb_read (delay, a, q);

		if (d_exp !== q)
			$display("Data compare error. Received %h, expected %h at time %t", q, d_exp, $time);
	end
endtask

endmodule


