
`timescale 1ns / 1ps

module cfg_master_model #(
	parameter ADDR_WIDTH = 16,
	parameter DATA_WIDTH = 32
)(
	input                       cfg_clk_i ,    
	input                       cfg_rstn_i,    
	output reg                  cfg_wren_o,    
	output reg [ADDR_WIDTH-1:0] cfg_addr_o,    
	output reg                  cfg_rden_o,    
	output reg [DATA_WIDTH-1:0] cfg_wdat_o,    
	input      [DATA_WIDTH-1:0] cfg_rdat_i,    
	input                       cfg_rdat_vld_i,
	input                       cfg_wdat_rdy_i 
);


////////////////////////////////////////////////////////////////////
//
// Memory Logic
//

initial
	begin
		cfg_addr_o  = {ADDR_WIDTH{1'b0}};
		cfg_wdat_o  = {DATA_WIDTH{1'b0}};
		cfg_wren_o  = 1'h0;
		cfg_rden_o  = 1'h0;
		#1;
		$display("\nINFO: CFG MASTER MODEL INSTANTIATED (%m)\n");
	end

////////////////////////////////////////////////////////////////////
//
// CFG write cycle
//

task cfg_write;
	input   delay;
	integer delay;

	input	[ADDR_WIDTH -1:0]	a;
	input	[DATA_WIDTH -1:0]	d;

	begin
		repeat(delay) @(posedge cfg_clk_i);
		#1;
		cfg_addr_o   = a;
		cfg_wdat_o   = d;
		cfg_wren_o   = 1'b1;
		@(posedge cfg_clk_i);
		while(~cfg_wdat_rdy_i)	@(posedge cfg_clk_i);
		#1;
		cfg_addr_o  = {ADDR_WIDTH{1'b0}};
		cfg_wdat_o  = {DATA_WIDTH{1'b0}};
		cfg_wren_o  = 1'h0;
		@(posedge cfg_clk_i);
		@(posedge cfg_clk_i);

	end
endtask


task cfg_read;
	input   delay;
	integer delay;

	input	[ADDR_WIDTH -1:0]	a;
	output	[DATA_WIDTH -1:0]	d;

	begin

		repeat(delay) @(posedge cfg_clk_i);

		#1;
		cfg_addr_o  = a;
		cfg_wdat_o  = {DATA_WIDTH{1'b0}};
		cfg_rden_o  = 1'b1;

		@(posedge cfg_clk_i);

		while(~cfg_rdat_vld_i)	@(posedge cfg_clk_i);

		#1;
		cfg_addr_o  = {ADDR_WIDTH{1'b0}};
		cfg_wdat_o  = {DATA_WIDTH{1'b0}};
		cfg_rden_o  = 1'h0;
		d    = cfg_rdat_i;
		@(posedge cfg_clk_i);
		@(posedge cfg_clk_i);
		

	end
endtask

reg [DATA_WIDTH   -1:0] q;

task cfg_cmp;
	input   delay;
	integer delay;

	input [ADDR_WIDTH -1:0]	a;
	input [DATA_WIDTH -1:0]	d_exp;

	begin
		cfg_read (delay, a, q);

		if (d_exp !== q)
			$display("Data compare error. Received %h, expected %h at time %t", q, d_exp, $time);
	end
endtask

endmodule


