

`timescale 1ns / 1ps

module demo_tb ;


parameter NUM_OF_NL_GPIO  = 32;
parameter NUM_OF_UART_DEV = 1;
parameter NUM_OF_SMBUS_DEV = 1;

reg apb_clk_i = 0;
reg apb_rstn_i = 0;

	reg   [                    15 :0]       scm_ll_gpio_in = 16'd1;
	wire  [                    15 :0]       scm_ll_gpio_out;

	wire  [  (NUM_OF_NL_GPIO - 1) :0]       scm_nl_gpio_in; 
	assign scm_nl_gpio_in[0] = 1;
	assign scm_nl_gpio_in[30:1] = 'h0;
	assign scm_nl_gpio_in[31] = 1;
	wire  [  (NUM_OF_NL_GPIO - 1) :0]       scm_nl_gpio_out;
	
    wire     [ (NUM_OF_UART_DEV - 1) :0]       scm_uart_rxd;          //UART interfaces tunneling through LVDS
    reg     [ (NUM_OF_UART_DEV - 1) :0]       scm_uart_cts = 2'd0;          //Clear To Send
    wire    [ (NUM_OF_UART_DEV - 1) :0]       scm_uart_txd;
    wire    [ (NUM_OF_UART_DEV - 1) :0]       scm_uart_rts;          //Request To Send

	wire     [(NUM_OF_SMBUS_DEV - 1) :0]     scm_smb_scl;
	wire     [(NUM_OF_SMBUS_DEV - 1) :0]     scm_smb_sda;
	
	reg   [                    31 :0]       scm_oem_gpio_in = 32'h123;
	wire  [                    31 :0]       scm_oem_gpio_out;		
	

	wire      [31:0]   scm_apb_addr   ;
	wire               scm_apb_sel    ;
	wire               scm_apb_ena    ;
	wire               scm_apb_wr     ;
	wire     [31:0]    scm_apb_rdata   ;
	wire     [31:0]    scm_apb_wdata   ;
	wire               scm_apb_rdy     ;
	wire               scm_apb_err     ;
	wire     [3 :0]    scm_apb_strb    ;
	
	
/////////////////////////////////////////////

	

	reg   [                    15 :0]       hpm_ll_gpio_in = 16'd3;
	wire  [                    15 :0]       hpm_ll_gpio_out;

	reg   [  (NUM_OF_NL_GPIO - 1) :0]       hpm_nl_gpio_in = 32'd5;    
	wire  [  (NUM_OF_NL_GPIO - 1) :0]       hpm_nl_gpio_out;

    reg    [ (NUM_OF_UART_DEV - 1) :0]        hpm_uart_rxd = 3'h7;          //UART interfaces tunneling through LVDS
    reg     [ (NUM_OF_UART_DEV - 1) :0]       hpm_uart_cts = 2'd0;            //Clear To Send
    wire    [ (NUM_OF_UART_DEV - 1) :0]       hpm_uart_txd;
    wire    [ (NUM_OF_UART_DEV - 1) :0]       hpm_uart_rts;          //Request To Send
	
	wire     [(NUM_OF_SMBUS_DEV - 1) :0] hpm_smb_scl;
	wire     [(NUM_OF_SMBUS_DEV - 1) :0] hpm_smb_sda;		
	
	reg   [                    31 :0]       hpm_oem_gpio_in = 32'h456;
	wire  [                    31 :0]       hpm_oem_gpio_out;		
	
	wire      [31:0]   hpm_apb_addr   ;  //master
	wire               hpm_apb_sel    ;
	wire               hpm_apb_ena    ;
	wire               hpm_apb_wr     ;
	wire      [31:0]   hpm_apb_rdata  ;
	wire      [31:0]   hpm_apb_wdata  ;
	wire               hpm_apb_rdy    ;
	wire               hpm_apb_err    ;
	wire      [3 :0]   hpm_apb_strb   ;	
	
	
	
	wire [3:0] HPM_local_link_state ;
	wire [3:0] HPM_remote_link_state;
	wire [3:0] SCM_local_link_state ;
	wire [3:0] SCM_remote_link_state;


	wire     scm_smb_scl_watch = scm_smb_scl[0];
	wire     scm_smb_sda_watch = scm_smb_sda[0];
	
	wire     hpm_smb_scl_watch = hpm_smb_scl[0];
	wire     hpm_smb_sda_watch = hpm_smb_sda[0];	

    reg       [7 :0]   scm_data_channel_tag_i = 1;
    wire      [7 :0]   scm_data_channel_tag_o;

    reg       [7 :0]   hpm_data_channel_tag_i = 2;
    wire      [7 :0]   hpm_data_channel_tag_o;	
	
	
scm_hpm_system_top  u_scm_hpm_system_top(
	.apb_clk_i             (apb_clk_i       ),
	.apb_rstn_i            (apb_rstn_i      ),
	.scm_ll_gpio_in        (scm_ll_gpio_in  ),
	.scm_ll_gpio_out       (scm_ll_gpio_out ),
	.scm_nl_gpio_in        (scm_nl_gpio_in  ), 
	.scm_nl_gpio_out       (scm_nl_gpio_out ),	
    .scm_uart_rxd          (scm_uart_rxd    ),  //      
    .scm_uart_cts          (scm_uart_cts    ),  //        
    .scm_uart_txd          (scm_uart_txd    ), 
    .scm_uart_rts          (scm_uart_rts    ),        
	.scm_smb_scl           (scm_smb_scl     ),
	.scm_smb_sda           (scm_smb_sda     ),
    .scm_oem_gpio_in       (scm_oem_gpio_in ),
    .scm_oem_gpio_out      (scm_oem_gpio_out),  
	                       
	.scm_apb_addr          (scm_apb_addr ), //input 
	.scm_apb_sel           (scm_apb_sel  ), //input
	.scm_apb_ena           (scm_apb_ena  ), //input
	.scm_apb_wr            (scm_apb_wr   ), //input
	.scm_apb_rdata         (scm_apb_rdata), //output
	.scm_apb_wdata         (scm_apb_wdata), //input
	.scm_apb_rdy           (scm_apb_rdy  ), //output
	.scm_apb_err           (scm_apb_err  ),
	.scm_apb_strb          (scm_apb_strb ), //input		
    .scm_data_channel_tag_i(scm_data_channel_tag_i),
    .scm_data_channel_tag_o(scm_data_channel_tag_o),
	
	.hpm_ll_gpio_in     (hpm_ll_gpio_in ),
	.hpm_ll_gpio_out    (hpm_ll_gpio_out),
	.hpm_nl_gpio_in     (hpm_nl_gpio_in ),    
	.hpm_nl_gpio_out    (hpm_nl_gpio_out),
    .hpm_uart_rxd       (hpm_uart_rxd   ),        
    .hpm_uart_cts       (hpm_uart_cts   ),        
    .hpm_uart_txd       (hpm_uart_txd   ),
    .hpm_uart_rts       (hpm_uart_rts   ),
	
	.hpm_smb_scl        (hpm_smb_scl     ),
	.hpm_smb_sda        (hpm_smb_sda     ),		
    .hpm_oem_gpio_in    (hpm_oem_gpio_in ),
    .hpm_oem_gpio_out   (hpm_oem_gpio_out),  
	
	.hpm_apb_addr       (hpm_apb_addr   ), //output
	.hpm_apb_sel        (hpm_apb_sel    ), //output
	.hpm_apb_ena        (hpm_apb_ena    ), //output
	.hpm_apb_wr         (hpm_apb_wr     ), //output
	.hpm_apb_rdata      (hpm_apb_rdata  ), //input
	.hpm_apb_wdata      (hpm_apb_wdata  ), //output
	.hpm_apb_rdy        (hpm_apb_rdy),//(hpm_apb_rdy    ), //input
	.hpm_apb_err        (hpm_apb_err  ),  //input
	.hpm_apb_strb       (hpm_apb_strb   ), //output		
    .hpm_data_channel_tag_i(hpm_data_channel_tag_i),
    .hpm_data_channel_tag_o(hpm_data_channel_tag_o)
);
	
assign hpm_apb_err = 0;

////////////////////////////////////////////////////////
	
reg sim_start = 0;	

	
uart_lib u_uart_lib (

	.uart_rx_o (scm_uart_rxd[0]),  
	.uart_tx_i (hpm_uart_txd[0]),  
	.sim_start_i(sim_start)


);
	
//assign scm_uart_rxd[1] = scm_uart_rxd[0];	


//assign scm_uart_rxd[1:0] = 0;			

////////////////////////////////////////////////////////

reg                         cfg_clk_i  = 0;    
reg                         cfg_rstn_i = 0;    
wire                        cfg_wren;    
wire [3-1:0]                cfg_addr;    
wire                        cfg_rden;    
wire [8-1:0]                cfg_wdat;    
wire [8-1:0]                cfg_rdat;    
wire                        cfg_rdat_vld;
wire                        cfg_wdat_rdy;


always #20 apb_clk_i = ~apb_clk_i;

initial begin
	apb_rstn_i = 0;
	#100;
	apb_rstn_i = 1;
end



always #5 cfg_clk_i = ~cfg_clk_i;

initial begin
	cfg_rstn_i = 0;
	#100;
	cfg_rstn_i = 1;
end



cfg_master_model #(
	.ADDR_WIDTH(3),
	.DATA_WIDTH(8)
)u0(
	.cfg_clk_i     (cfg_clk_i    ),    
	.cfg_rstn_i    (cfg_rstn_i   ),    
	.cfg_wren_o    (cfg_wren     ),    
	.cfg_addr_o    (cfg_addr     ),    
	.cfg_rden_o    (cfg_rden     ),    
	.cfg_wdat_o    (cfg_wdat     ),    
	.cfg_rdat_i    (cfg_rdat     ),    
	.cfg_rdat_vld_i(cfg_rdat_vld ),
	.cfg_wdat_rdy_i(cfg_wdat_rdy ) 
);
	

apb_master_model #(
	.ADDR_WIDTH(32),
	.DATA_WIDTH(32)
)u1(
	.apb_clk_i   (apb_clk_i     ),    
	.apb_rstn_i  (apb_rstn_i    ),   
	.apb_addr_o  (scm_apb_addr  ),  
	.apb_sel_o   (scm_apb_sel   ),
	.apb_ena_o   (scm_apb_ena   ),
	.apb_wr_o    (scm_apb_wr    ),    
	.apb_wdata_o (scm_apb_wdata ),    
	.apb_rdata_i (scm_apb_rdata ),    
	.apb_rdy_i   (scm_apb_rdy   ),
	.apb_strb_o  (scm_apb_strb  ) 
);	

wire           local_wren_w;
wire [31:0]    local_addr_w;
wire           local_rden_w;
wire [31:0]    local_wdat_w;
wire [31:0]    local_rdat_w;
wire [3 :0]    local_strb_w;
wire           local_rdat_vld_w;
wire           local_wdat_rdy_w;

apb2local u_apb2local
(

	.cfg_clk_i  (apb_clk_i      ),
	.cfg_rstn_i (apb_rstn_i     ),
	.cfg_addr_i (hpm_apb_addr   ),
	.cfg_sel_i  (hpm_apb_sel    ),
	.cfg_ena_i  (hpm_apb_ena    ),
	.cfg_wr_i   (hpm_apb_wr     ),
	.cfg_rdata_o(hpm_apb_rdata  ),
	.cfg_wdata_i(hpm_apb_wdata  ),
	.cfg_rdy_o  (hpm_apb_rdy    ),
	.cfg_strb_i (hpm_apb_strb   ),
	
	.local_wren_o(local_wren_w),
	.local_addr_o(local_addr_w),
	.local_strb_o(local_strb_w),
	.local_rden_o(local_rden_w),
	.local_wdat_o(local_wdat_w),
	.local_rdat_i(local_rdat_w),
	.local_rdat_vld_i(local_rdat_vld_w),
	.local_wdat_rdy_i(local_wdat_rdy_w)


);

wire [31:0] reg0x0000;wire [31:0] reg0x0010;wire [31:0] mon0x0080;wire [31:0] mon0x0090;
wire [31:0] reg0x0001;wire [31:0] reg0x0011;wire [31:0] mon0x0081;wire [31:0] mon0x0091;
wire [31:0] reg0x0002;wire [31:0] reg0x0012;wire [31:0] mon0x0082;wire [31:0] mon0x0092;
wire [31:0] reg0x0003;wire [31:0] reg0x0013;wire [31:0] mon0x0083;wire [31:0] mon0x0093;
wire [31:0] reg0x0004;wire [31:0] reg0x0014;wire [31:0] mon0x0084;wire [31:0] mon0x0094;
wire [31:0] reg0x0005;wire [31:0] reg0x0015;wire [31:0] mon0x0085;wire [31:0] mon0x0095;
wire [31:0] reg0x0006;wire [31:0] reg0x0016;wire [31:0] mon0x0086;wire [31:0] mon0x0096;
wire [31:0] reg0x0007;wire [31:0] reg0x0017;wire [31:0] mon0x0087;wire [31:0] mon0x0097;
wire [31:0] reg0x0008;wire [31:0] reg0x0018;wire [31:0] mon0x0088;wire [31:0] mon0x0098;
wire [31:0] reg0x0009;wire [31:0] reg0x0019;wire [31:0] mon0x0089;wire [31:0] mon0x0099;
wire [31:0] reg0x000A;wire [31:0] reg0x001A;wire [31:0] mon0x008A;wire [31:0] mon0x009A;
wire [31:0] reg0x000B;wire [31:0] reg0x001B;wire [31:0] mon0x008B;wire [31:0] mon0x009B;
wire [31:0] reg0x000C;wire [31:0] reg0x001C;wire [31:0] mon0x008C;wire [31:0] mon0x009C;
wire [31:0] reg0x000D;wire [31:0] reg0x001D;wire [31:0] mon0x008D;wire [31:0] mon0x009D;
wire [31:0] reg0x000E;wire [31:0] reg0x001E;wire [31:0] mon0x008E;wire [31:0] mon0x009E;
wire [31:0] reg0x000F;wire [31:0] reg0x001F;wire [31:0] mon0x008F;wire [31:0] mon0x009F;


local2reg u_local2reg
(

	.cfg_clk_i(apb_clk_i),
	.cfg_rstn_i(apb_rstn_i),
	.local_wren_i(local_wren_w),
	.local_addr_i(local_addr_w),
	.local_strb_i(4'hf),	
	.local_rden_i(local_rden_w),
	.local_wdat_i(local_wdat_w),
	.local_rdat_o(local_rdat_w),
	.local_rdat_vld_o(local_rdat_vld_w),
	.local_wdat_rdy_o(local_wdat_rdy_w),	
	
	
	.reg0x0000(reg0x0000),.reg0x0010(reg0x0010),.mon0x0080(mon0x0080),.mon0x0090(mon0x0090),
	.reg0x0001(reg0x0001),.reg0x0011(reg0x0011),.mon0x0081(mon0x0081),.mon0x0091(mon0x0091),
	.reg0x0002(reg0x0002),.reg0x0012(reg0x0012),.mon0x0082(mon0x0082),.mon0x0092(mon0x0092),
    .reg0x0003(reg0x0003),.reg0x0013(reg0x0013),.mon0x0083(mon0x0083),.mon0x0093(mon0x0093),
    .reg0x0004(reg0x0004),.reg0x0014(reg0x0014),.mon0x0084(mon0x0084),.mon0x0094(mon0x0094),
    .reg0x0005(reg0x0005),.reg0x0015(reg0x0015),.mon0x0085(mon0x0085),.mon0x0095(mon0x0095),
    .reg0x0006(reg0x0006),.reg0x0016(reg0x0016),.mon0x0086(mon0x0086),.mon0x0096(mon0x0096),
    .reg0x0007(reg0x0007),.reg0x0017(reg0x0017),.mon0x0087(mon0x0087),.mon0x0097(mon0x0097),
    .reg0x0008(reg0x0008),.reg0x0018(reg0x0018),.mon0x0088(mon0x0088),.mon0x0098(mon0x0098),
    .reg0x0009(reg0x0009),.reg0x0019(reg0x0019),.mon0x0089(mon0x0089),.mon0x0099(mon0x0099),
    .reg0x000A(reg0x000A),.reg0x001A(reg0x001A),.mon0x008A(mon0x008A),.mon0x009A(mon0x009A),
    .reg0x000B(reg0x000B),.reg0x001B(reg0x001B),.mon0x008B(mon0x008B),.mon0x009B(mon0x009B),
    .reg0x000C(reg0x000C),.reg0x001C(reg0x001C),.mon0x008C(mon0x008C),.mon0x009C(mon0x009C),
    .reg0x000D(reg0x000D),.reg0x001D(reg0x001D),.mon0x008D(mon0x008D),.mon0x009D(mon0x009D),
    .reg0x000E(reg0x000E),.reg0x001E(reg0x001E),.mon0x008E(mon0x008E),.mon0x009E(mon0x009E),
    .reg0x000F(reg0x000F),.reg0x001F(reg0x001F),.mon0x008F(mon0x008F),.mon0x009F(mon0x009F)   
	
	
	
);






reg [31:0]channel_reg;

	
initial
  begin
	#3000000;

	// program internal registers
	u1.apb_write(1, 32'h00000004, 32'h12345678); // load prescaler lo-byte
	
	#10000;
	u1.apb_read(1, 32'h00000004, channel_reg);
  end
	
wire scl_pad_i;
wire scl_pad_o;
wire scl_padoen_o;
wire sda_pad_i;
wire sda_pad_o;
wire sda_padoen_o;


i2c_master_cfgport u_i2c_master_cfgport(
	.cfg_clk_i      (cfg_clk_i           ), 
	.cfg_rstn_i     (cfg_rstn_i          ), 
	.cfg_wren_i     (cfg_wren            ), 
	.cfg_addr_i     (cfg_addr            ), 
	.cfg_rden_i     (cfg_rden            ),
	.cfg_wdat_i     (cfg_wdat            ), 
	.cfg_rdat_o     (cfg_rdat            ), 
	.cfg_rdat_vld_o (cfg_rdat_vld        ), 
	.cfg_wdat_rdy_o (cfg_wdat_rdy        ), 
	.cfg_inta_o     (                    ),
	.scl_io         (scm_smb_scl[0]      ), 
	.sda_io         (scm_smb_sda[0]      )
);





pullup p1(scm_smb_scl[0]); // pullup scl line
pullup p2(scm_smb_sda[0]); // pullup sda line
pullup p3(hpm_smb_scl[0]); // pullup scl line
pullup p4(hpm_smb_sda[0]); // pullup sda line

	
    GSR GSR(.GSRI(1'b1));
    
i2c_slave_model u_i2c_slave_model
(
	.scl(hpm_smb_scl[0]),
	.sda(hpm_smb_sda[0])
);




parameter PRER_LO = 3'b000;
parameter PRER_HI = 3'b001;
parameter CTR     = 3'b010;
parameter RXR     = 3'b011;
parameter TXR     = 3'b011;
parameter CR      = 3'b100;
parameter SR      = 3'b100;

parameter TXR_R   = 3'b101; 
parameter CR_R    = 3'b110; 

parameter RD      = 1'b1;
parameter WR      = 1'b0;
parameter SADR    = 7'b1010_000;

reg [7:0] q, qq;

initial
  begin
	#3000000;

	// program internal registers
	u0.cfg_write(1, PRER_LO, 8'hc8); // load prescaler lo-byte
	u0.cfg_write(1, PRER_HI, 8'h00); // load prescaler hi-byte
	$display("status: %t programmed registers", $time);

	u0.cfg_cmp(0, PRER_LO, 8'hc8); // verify prescaler lo-byte
	u0.cfg_cmp(0, PRER_HI, 8'h00); // verify prescaler hi-byte
	$display("status: %t verified registers", $time);

	u0.cfg_write(1, CTR,   8'h80); // enable core
	$display("status: %t core enabled", $time);

	//
	// access slave (write)
	//

	// drive slave address
	u0.cfg_write(1, TXR, {SADR,WR} ); // present slave address, set write-bit
	u0.cfg_write(0, CR,      8'h90 ); // set command (start, write)
	$display("status: %t generate 'start', write cmd %0h (slave address+write)", $time, {SADR,WR} );

	// check tip bit
	u0.cfg_read(1, SR, q);
	while(q[1])
	   u0.cfg_read(0, SR, q); // poll it until it is zero
	$display("status: %t tip==0", $time);

	
	//#10000000;
	
	
	// send memory address
	u0.cfg_write(1, TXR,     8'h01); // present slave's memory address
	u0.cfg_write(0, CR,      8'h10); // set command (write)
	$display("status: %t write slave memory address 01", $time);

	// check tip bit
	u0.cfg_read(1, SR, q);
	while(q[1])
	   u0.cfg_read(0, SR, q); // poll it until it is zero
	$display("status: %t tip==0", $time);

	// send memory contents
	u0.cfg_write(1, TXR,     8'ha5); // present data
	u0.cfg_write(0, CR,      8'h10); // set command (write)
	$display("status: %t write data a5", $time);
	

	u0.cfg_read(1, SR, q);
	while(q[1])
	   u0.cfg_read(1, SR, q); // poll it until it is zero
	$display("status: %t tip==0", $time);

	// send memory contents for next memory address (auto_inc)
	u0.cfg_write(1, TXR,     8'h5a); // present data
	u0.cfg_write(0, CR,      8'h50); // set command (stop, write)
	$display("status: %t write next data 5a, generate 'stop'", $time);

	u0.cfg_read(1, SR, q);
	while(q[1])
	   u0.cfg_read(1, SR, q); // poll it until it is zero
	$display("status: %t tip==0", $time);		
	
	
	// access slave (read)
	//

	// drive slave address
	u0.cfg_write(1, TXR,{SADR,WR} ); // present slave address, set write-bit
	u0.cfg_write(0, CR,     8'h90 ); // set command (start, write)
	$display("status: %t generate 'start', write cmd %0h (slave address+write)", $time, {SADR,WR} );

	// check tip bit
	u0.cfg_read(1, SR, q);
	while(q[1])
	   u0.cfg_read(1, SR, q); // poll it until it is zero
	$display("status: %t tip==0", $time);
	
	// send memory address
	u0.cfg_write(1, TXR,     8'h01); // present slave's memory address
	u0.cfg_write(0, CR,      8'h10); // set command (write)
	$display("status: %t write slave address 01", $time);	
	
	// check tip bit
	u0.cfg_read(1, SR, q);
	while(q[1])
	   u0.cfg_read(1, SR, q); // poll it until it is zero
	$display("status: %t tip==0", $time);
	
	
	
	
	
	
	
	      // drive slave address
	      u0.cfg_write(1, TXR, {SADR,RD} ); // present slave's address, set read-bit
	      u0.cfg_write(0, CR,      8'h90 ); // set command (start, write)
	      $display("status: %t generate 'repeated start', write cmd %0h (slave address+read)", $time, {SADR,RD} );

	      // check tip bit
	      u0.cfg_read(1, SR, q);
	      while(q[1])
	           u0.cfg_read(1, SR, q); // poll it until it is zero
	      $display("status: %t tip==0", $time);

	      // read data from slave
	      u0.cfg_write(1, CR,      8'h20); // set command (read, ack_read)
	      $display("status: %t read + ack", $time);
			
			
		
			
	      // check tip bit
	      u0.cfg_read(1, SR, q);
	      while(q[1])
	           u0.cfg_read(1, SR, q); // poll it until it is zero
	      $display("status: %t tip==0", $time);

	      // check data just received
	      u0.cfg_read(1, RXR, qq);
	      if(qq !== 8'ha5)
	        $display("\nERROR: Expected a5, received %x at time %t", qq, $time);
	      else
	        $display("status: %t received %x", $time, qq);

	      // read data from slave
	      u0.cfg_write(1, CR,      8'h28); // set command (read, ack_read)
	      $display("status: %t read + ack", $time);

	      // check tip bit
	      u0.cfg_read(1, SR, q);
	      while(q[1])
	           u0.cfg_read(1, SR, q); // poll it until it is zero
	      $display("status: %t tip==0", $time);

	      // check data just received
	      u0.cfg_read(1, RXR, qq);
	      if(qq !== 8'h5a)
	        $display("\nERROR: Expected 5a, received %x at time %t", qq, $time);
	      else
	        $display("status: %t received %x", $time, qq);

			
			
			//#100000;

	      // read data from slave
	      u0.cfg_write(1, CR,      8'h40); // set command (stop)
	      $display("status: %t generate 'stop'", $time);

	      // check tip bit
	      u0.cfg_read(1, SR, q);
	      while(q[1])
	      u0.cfg_read(1, SR, q); // poll it until it is zero
	      $display("status: %t tip==0", $time);

	      #250000; // wait 250us
	      $display("\n\nstatus: %t Testbench done", $time);
	
	
	
	
	
	
	
	
	
	
	
	
	
end















	
	
	
	
endmodule //testbench
