

`timescale 1ns / 1ps

module scm_hpm_system_top #(

	parameter NUM_OF_NL_GPIO                   = 32,
	parameter NUM_OF_UART_DEV                  = 1, // from 1 to 2
	parameter NUM_OF_SMBUS_DEV                 = 1
	

)(

	input apb_clk_i,
	input apb_rstn_i,


	input   [                    15 :0]       scm_ll_gpio_in,
	output  [                    15 :0]       scm_ll_gpio_out,

	input   [  (NUM_OF_NL_GPIO - 1) :0]       scm_nl_gpio_in, 
	output  [  (NUM_OF_NL_GPIO - 1) :0]       scm_nl_gpio_out,
	
    input     [ (NUM_OF_UART_DEV - 1) :0]       scm_uart_rxd,           //UART interfaces tunneling through LVDS
    input     [ (NUM_OF_UART_DEV - 1) :0]       scm_uart_cts,           //Clear To Send
    output    [ (NUM_OF_UART_DEV - 1) :0]       scm_uart_txd,
    output    [ (NUM_OF_UART_DEV - 1) :0]       scm_uart_rts,           //Request To Send
	
	inout     [(NUM_OF_SMBUS_DEV - 1) :0] scm_smb_scl,
	inout     [(NUM_OF_SMBUS_DEV - 1) :0] scm_smb_sda,
	
	input   [                    31 :0]       scm_oem_gpio_in,
	output  [                    31 :0]       scm_oem_gpio_out,	
	
	input      [31:0]   scm_apb_addr   ,  //slave
	input               scm_apb_sel    ,
	input               scm_apb_ena    ,
	input               scm_apb_wr     ,
	output     [31:0]   scm_apb_rdata  ,
	input      [31:0]   scm_apb_wdata  ,
	output              scm_apb_rdy    ,
	output              scm_apb_err    ,
	input      [3 :0]   scm_apb_strb   ,	
	
    input       [7 :0]   scm_data_channel_tag_i,
    output      [7 :0]   scm_data_channel_tag_o,
	
/////////////////////////////////////////////

	

	input   [                    15 :0]       hpm_ll_gpio_in,
	output  [                    15 :0]       hpm_ll_gpio_out,

	input   [  (NUM_OF_NL_GPIO - 1) :0]       hpm_nl_gpio_in,    
	output  [  (NUM_OF_NL_GPIO - 1) :0]       hpm_nl_gpio_out,

    input     [ (NUM_OF_UART_DEV - 1) :0]     hpm_uart_rxd,           //UART interfaces tunneling through LVDS
    input     [ (NUM_OF_UART_DEV - 1) :0]     hpm_uart_cts,           //Clear To Send
    output    [ (NUM_OF_UART_DEV - 1) :0]     hpm_uart_txd,
    output    [ (NUM_OF_UART_DEV - 1) :0]     hpm_uart_rts,           //Request To Send
	
	inout     [(NUM_OF_SMBUS_DEV - 1) :0]     hpm_smb_scl,
	inout     [(NUM_OF_SMBUS_DEV - 1) :0]     hpm_smb_sda,	
	
	input   [                    31 :0]       hpm_oem_gpio_in,
	output  [                    31 :0]       hpm_oem_gpio_out,		
	
	output      [31:0]   hpm_apb_addr   ,  //master
	output               hpm_apb_sel    ,
	output               hpm_apb_ena    ,
	output               hpm_apb_wr     ,
	input       [31:0]   hpm_apb_rdata  ,
	output      [31:0]   hpm_apb_wdata  ,
	input                hpm_apb_rdy    ,
	input                hpm_apb_err    ,
	output      [3 :0]   hpm_apb_strb   ,	

    input       [7 :0]   hpm_data_channel_tag_i,
    output      [7 :0]   hpm_data_channel_tag_o
		
);





reg                   ref_clk  = 0        ;  //25M
reg                   clk      = 0        ;  //60M
reg                   rstn_i   = 0        ;

wire                 scm_lvds_tx_data;
wire                 scm_lvds_tx_clk ; 
wire                 scm_lvds_rx_data;
wire                 scm_lvds_rx_clk ;

wire                 scm_tx_pll_locked ;
wire                 scm_aligned       ;            //Mark that LVDS link has locked
wire                 scm_NL_gpio_stable; 




wire                 hpm_lvds_tx_data;
wire                 hpm_lvds_tx_clk ; 
wire                 hpm_lvds_rx_data;
wire                 hpm_lvds_rx_clk ;

wire                 hpm_tx_pll_locked;
wire                 hpm_aligned;            //Mark that LVDS link has locked
wire                 hpm_NL_gpio_stable; 


reg scmrstn_i = 0;
reg hpmrstn_i = 0;

always #8.3          clk     = ~clk;     //60M
always #19.92         ref_clk = ~ref_clk; //25M

initial begin
	rstn_i = 0;
	#1000;
	rstn_i = 1;
	scmrstn_i = 0;
	hpmrstn_i = 0;
	#1000;
	scmrstn_i = 1;
	hpmrstn_i = 1;


end




DCSCM_LTPI_Top u_ltpi_scm
(
	.clk_i            (clk                 ),       
	.ref_clk_i        (ref_clk             ),       
	.rstn_i           (rstn_i              ),
    .start_i          (scmrstn_i           ),
	.lvds_tx_data_o   (scm_lvds_tx_data    ),
	.lvds_tx_clk_o    (scm_lvds_tx_clk     ),
	.lvds_rx_data_i   (scm_lvds_rx_data    ),
	.lvds_rx_clk_i    (scm_lvds_rx_clk     ),
	.tx_pll_locked_o  (scm_tx_pll_locked   ),
	.aligned_o        (scm_aligned         ),    
	.ll_gpio_in_i     (scm_ll_gpio_in      ),
	.ll_gpio_out_o    (scm_ll_gpio_out     ),
	.nl_gpio_in_i     (scm_nl_gpio_in      ),       
	.nl_gpio_out_o    (scm_nl_gpio_out     ),
	.uart_rxd_i       (scm_uart_rxd        ),       
	//.uart_cts_i       (scm_uart_cts        ),     
	.uart_txd_o       (scm_uart_txd        ),
	//.uart_rts_o       (scm_uart_rts        ),     
	.iic_scl_io       (scm_smb_scl         ),       
	.iic_sda_io       (scm_smb_sda         ),
    .oem_gpio_in_i    (scm_oem_gpio_in     ),
    .oem_gpio_out_o   (scm_oem_gpio_out    ),  

	.apb_addr_i       (scm_apb_addr        ),  
	.apb_sel_i        (scm_apb_sel         ),
	.apb_ena_i        (scm_apb_ena         ),
	.apb_wr_i         (scm_apb_wr          ),
	.apb_rdata_o      (scm_apb_rdata       ),
	.apb_wdata_i      (scm_apb_wdata       ),
	.apb_rdy_o        (scm_apb_rdy         ),
	.apb_err_o        (scm_apb_err         ),
	.apb_strb_i       (scm_apb_strb        ),
    .data_channel_tag_i(scm_data_channel_tag_i),
    .data_channel_tag_o(scm_data_channel_tag_o),

	.cfg_clk_i        (apb_clk_i           ),
	.cfg_rstn_i       (apb_rstn_i          ),
	.cfg_addr_i       (0                   ),
	.cfg_sel_i        (0                   ),
	.cfg_ena_i        (0                   ),
	.cfg_wr_i         (0                   ),
	.cfg_rdata_o      (                    ),
	.cfg_wdata_i      (0                   ),
	.cfg_rdy_o        (                    ),
	.cfg_strb_i       (0                   )
);
	

LTPI_HPM_Top u_ltpi_hpm
(
	.clk_i            (clk                 ),       
	.ref_clk_i        (ref_clk             ),       
	.rstn_i           (rstn_i              ),
    .start_i          (hpmrstn_i           ),    
	.lvds_tx_data_o   (hpm_lvds_tx_data    ),
	.lvds_tx_clk_o    (hpm_lvds_tx_clk     ),
	.lvds_rx_data_i   (hpm_lvds_rx_data    ),
	.lvds_rx_clk_i    (hpm_lvds_rx_clk     ),
	.tx_pll_locked_o  (hpm_tx_pll_locked   ),
	.aligned_o        (hpm_aligned         ),    
	.ll_gpio_in_i     (hpm_ll_gpio_in      ),
	.ll_gpio_out_o    (hpm_ll_gpio_out     ),
	.nl_gpio_in_i     (hpm_nl_gpio_in      ),       
	.nl_gpio_out_o    (hpm_nl_gpio_out     ),
	.uart_rxd_i       (hpm_uart_rxd        ),       
	//.uart_cts_i       (hpm_uart_cts        ),     
	.uart_txd_o       (hpm_uart_txd        ),
	//.uart_rts_o       (hpm_uart_rts        ),     
	.iic_scl_io       (hpm_smb_scl         ),       
	.iic_sda_io       (hpm_smb_sda         ),
    .oem_gpio_in_i    (hpm_oem_gpio_in     ),
    .oem_gpio_out_o   (hpm_oem_gpio_out    ),  
	.apb_addr_o       (hpm_apb_addr        ), 
	.apb_sel_o        (hpm_apb_sel         ),
	.apb_ena_o        (hpm_apb_ena         ),
	.apb_wr_o         (hpm_apb_wr          ),
	.apb_rdata_i      (hpm_apb_rdata       ),
	.apb_wdata_o      (hpm_apb_wdata       ),
	.apb_rdy_i        (hpm_apb_rdy         ),
	.apb_err_i        (hpm_apb_err         ),
	.apb_strb_o       (hpm_apb_strb        ),	
    .data_channel_tag_i(hpm_data_channel_tag_i),
    .data_channel_tag_o(hpm_data_channel_tag_o),	
	
	.cfg_clk_i        (apb_clk_i           ),
	.cfg_rstn_i       (apb_rstn_i          ),
	.cfg_addr_i       (0                   ),
	.cfg_sel_i        (0                   ),
	.cfg_ena_i        (0                   ),
	.cfg_wr_i         (0                   ),
	.cfg_rdata_o      (                    ),
	.cfg_wdata_i      (0                   ),
	.cfg_rdy_o        (                    ),
	.cfg_strb_i       (0                   )
);	
	
	
	
	
	
	
assign  hpm_lvds_rx_data = scm_lvds_tx_data; 
assign  hpm_lvds_rx_clk  = scm_lvds_tx_clk;
assign  scm_lvds_rx_data = hpm_lvds_tx_data; 
assign  scm_lvds_rx_clk  = hpm_lvds_tx_clk;
	
	
	
	

	
	
	
	
endmodule //testbench
